@@ -53,6 +53,10 @@
 #include <libxfce4panel/xfce-panel-plugin.h>
 #include <libxfce4panel/xfce-panel-convenience.h>
 
+#ifdef HAVE_THUNAR_VFS
+#include <thunar-vfs/thunar-vfs.h>
+#endif
+
 #include "desktop-menu-stub.h"
 #include "xfdesktop-common.h"
 #include "xfce4-popup-menu.h"
@@ -61,9 +65,9 @@
 #define DEFAULT_BUTTON_ICON  DATADIR "/pixmaps/xfce4_xicon1.png"
 #define DEFAULT_BUTTON_TITLE "Xfce Menu"
 
-#ifndef CHECK_RUNNING_PLUGIN
-#define CHECK_RUNNING_PLUGIN 0
-#endif
+/* this'll only allow one copy of the plugin; useful for forcing the
+ * xfce4-popup-menu keybind to work with only one plugin */
+/* #define CHECK_RUNNING_PLUGIN */
 
 typedef struct _DMPlugin {
     XfcePanelPlugin *plugin;
@@ -89,24 +93,6 @@ typedef struct _DMPlugin {
 } DMPlugin;
 
 
-static gboolean
-dmp_allow_menu_customization(void)
-{
-    static gboolean allow_customization = FALSE;
-    static gboolean checked = FALSE;
-
-    if(G_UNLIKELY(!checked)) {
-        XfceKiosk *kiosk;
-        
-        kiosk = xfce_kiosk_new("xfdesktop");
-        allow_customization = xfce_kiosk_query(kiosk, "CustomizeDesktopMenu");
-        xfce_kiosk_free(kiosk);
-        checked = TRUE;
-    }
-
-    return allow_customization;
-}
-
 static gchar *
 dmp_get_real_path(const gchar *raw_path)
 {
@@ -899,13 +885,11 @@ dmp_create_options(XfcePanelPlugin *plugin, DMPlugin *dmp)
     gtk_widget_show(hbox);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
     
-    if(G_LIKELY(dmp_allow_menu_customization() == TRUE)) {
-        btn = xfce_create_mixed_button(GTK_STOCK_EDIT, _("_Edit Menu"));
-        gtk_widget_show(btn);
-        gtk_box_pack_end(GTK_BOX(hbox), btn, FALSE, FALSE, 0);
-        g_signal_connect(G_OBJECT(btn), "clicked",
-                G_CALLBACK(dmp_edit_menu_clicked_cb), dmp);
-    }
+    btn = xfce_create_mixed_button(GTK_STOCK_EDIT, _("_Edit Menu"));
+    gtk_widget_show(btn);
+    gtk_box_pack_end(GTK_BOX(hbox), btn, FALSE, FALSE, 0);
+    g_signal_connect(G_OBJECT(btn), "clicked",
+            G_CALLBACK(dmp_edit_menu_clicked_cb), dmp);
     
     frame = xfce_create_framebox(_("Icons"), &frame_bin);
     gtk_widget_show(frame);
@@ -952,10 +936,22 @@ dmp_create_options(XfcePanelPlugin *plugin, DMPlugin *dmp)
     gtk_widget_show(dlg);
 }
 
-#if CHECK_RUNNING_PLUGIN
-static gboolean 
-desktop_menu_plugin_check (GdkScreen *gscreen)
+#ifdef XFCE_PANEL_PLUGIN_REGISTER_EXTERNAL_FULL
+static gboolean
+desktop_menu_plugin_preinit(int argc,
+                            char **argv)
 {
+#ifdef HAVE_THUNAR_VFS
+    g_thread_init(NULL);
+#endif
+    return TRUE;
+}
+#endif
+
+static gboolean 
+desktop_menu_plugin_check(GdkScreen *gscreen)
+{
+#ifdef CHECK_RUNNING_PLUGIN
     gchar selection_name[32];
     Atom selection_atom;
     
@@ -966,13 +962,13 @@ desktop_menu_plugin_check (GdkScreen *gscreen)
     selection_atom = XInternAtom(GDK_DISPLAY(), selection_name, False);
 
     if(XGetSelectionOwner(GDK_DISPLAY(), selection_atom)) {
-        xfce_info (_("There is already a panel menu registered for this screen"));
+        xfce_info(_("There is already a panel menu registered for this screen"));
         return FALSE;
     }
+#endif
 
     return TRUE;
 }
-#endif
 
 static gboolean
 dmp_set_selection(DMPlugin *dmp)
@@ -1065,6 +1061,10 @@ desktop_menu_plugin_construct(XfcePanelPlugin *plugin)
     
     xfce_textdomain(GETTEXT_PACKAGE, LOCALEDIR, "UTF-8");
     
+#ifdef HAVE_THUNAR_VFS
+    thunar_vfs_init();
+#endif
+    
     if(!(dmp = dmp_new(plugin)))
         exit(1);
     
@@ -1072,16 +1072,14 @@ desktop_menu_plugin_construct(XfcePanelPlugin *plugin)
     gtk_container_add(GTK_CONTAINER(plugin), dmp->button);
     
     /* Add edit menu option to right click menu */
-    if(G_LIKELY(dmp_allow_menu_customization() == TRUE)) {
-        img = gtk_image_new_from_stock(GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU);
-        gtk_widget_show(img);
-        mi = gtk_image_menu_item_new_with_label(_("Edit Menu"));
-        gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi), img);
-        gtk_widget_show(mi);
-        xfce_panel_plugin_menu_insert_item(plugin, GTK_MENU_ITEM(mi));
-        g_signal_connect(G_OBJECT(mi), "activate", 
-                         G_CALLBACK(dmp_edit_menu_clicked_cb), dmp);
-    }
+    img = gtk_image_new_from_stock(GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU);
+    gtk_widget_show(img);
+    mi = gtk_image_menu_item_new_with_label(_("Edit Menu"));
+    gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi), img);
+    gtk_widget_show(mi);
+    xfce_panel_plugin_menu_insert_item(plugin, GTK_MENU_ITEM(mi));
+    g_signal_connect(G_OBJECT(mi), "activate", 
+                     G_CALLBACK(dmp_edit_menu_clicked_cb), dmp);
     
     g_signal_connect(plugin, "free-data",
                      G_CALLBACK(dmp_free), dmp);
@@ -1099,9 +1097,11 @@ desktop_menu_plugin_construct(XfcePanelPlugin *plugin)
     dmp_set_size(plugin, xfce_panel_plugin_get_size(plugin), dmp);
 }
 
-#if CHECK_RUNNING_PLUGIN
-XFCE_PANEL_PLUGIN_REGISTER_EXTERNAL_WITH_CHECK (desktop_menu_plugin_construct,
-                                                desktop_menu_plugin_check)
+#ifdef XFCE_PANEL_PLUGIN_REGISTER_EXTERNAL_FULL
+XFCE_PANEL_PLUGIN_REGISTER_EXTERNAL_FULL(desktop_menu_plugin_construct,
+                                         desktop_menu_plugin_preinit,
+                                         desktop_menu_plugin_check)
 #else
-XFCE_PANEL_PLUGIN_REGISTER_EXTERNAL(desktop_menu_plugin_construct)
+XFCE_PANEL_PLUGIN_REGISTER_EXTERNAL_WITH_CHECK(desktop_menu_plugin_construct,
+                                               desktop_menu_plugin_check)
 #endif
