@@ -14,8 +14,8 @@
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software Foundation,
- *  Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -27,24 +27,27 @@
 #endif
 
 #include <libwnck/libwnck.h>
-#include <libxfce4ui/libxfce4ui.h>
+#include <libwnck/window-action-menu.h>
+#include <libxfcegui4/libxfcegui4.h>
 
-#include "xfdesktop-common.h"
 #include "xfdesktop-window-icon.h"
 
 struct _XfdesktopWindowIconPrivate
 {
     gint workspace;
+    GdkPixbuf *pix;
+    gint cur_pix_size;
     gchar *label;
     WnckWindow *window;
 };
 
+static void xfdesktop_window_icon_class_init(XfdesktopWindowIconClass *klass);
+static void xfdesktop_window_icon_init(XfdesktopWindowIcon *icon);
 static void xfdesktop_window_icon_finalize(GObject *obj);
 
 static GdkPixbuf *xfdesktop_window_icon_peek_pixbuf(XfdesktopIcon *icon,
-                                                   gint width, gint height);
-static const gchar *xfdesktop_window_icon_peek_label(XfdesktopIcon *icon);
-static gchar *xfdesktop_window_icon_get_identifier(XfdesktopIcon *icon);
+                                                   gint size);
+static G_CONST_RETURN gchar *xfdesktop_window_icon_peek_label(XfdesktopIcon *icon);
 
 static gboolean xfdesktop_window_icon_activated(XfdesktopIcon *icon);
 static gboolean xfdesktop_window_icon_populate_context_menu(XfdesktopIcon *icon,
@@ -56,7 +59,7 @@ static void xfdesktop_window_icon_changed_cb(WnckWindow *window,
                                              gpointer user_data);
 
 
-G_DEFINE_TYPE_WITH_PRIVATE(XfdesktopWindowIcon, xfdesktop_window_icon, XFDESKTOP_TYPE_ICON)
+G_DEFINE_TYPE(XfdesktopWindowIcon, xfdesktop_window_icon, XFDESKTOP_TYPE_ICON)
 
 
 static void
@@ -64,12 +67,13 @@ xfdesktop_window_icon_class_init(XfdesktopWindowIconClass *klass)
 {
     GObjectClass *gobject_class = (GObjectClass *)klass;
     XfdesktopIconClass *icon_class = (XfdesktopIconClass *)klass;
-
+    
+    g_type_class_add_private(klass, sizeof(XfdesktopWindowIconPrivate));
+    
     gobject_class->finalize = xfdesktop_window_icon_finalize;
-
+    
     icon_class->peek_pixbuf = xfdesktop_window_icon_peek_pixbuf;
     icon_class->peek_label = xfdesktop_window_icon_peek_label;
-    icon_class->get_identifier = xfdesktop_window_icon_get_identifier;
     icon_class->activated = xfdesktop_window_icon_activated;
     icon_class->populate_context_menu = xfdesktop_window_icon_populate_context_menu;
 }
@@ -77,7 +81,8 @@ xfdesktop_window_icon_class_init(XfdesktopWindowIconClass *klass)
 static void
 xfdesktop_window_icon_init(XfdesktopWindowIcon *icon)
 {
-    icon->priv = xfdesktop_window_icon_get_instance_private(icon);
+    icon->priv = G_TYPE_INSTANCE_GET_PRIVATE(icon, XFDESKTOP_TYPE_WINDOW_ICON,
+                                             XfdesktopWindowIconPrivate);
 }
 
 static void
@@ -85,10 +90,13 @@ xfdesktop_window_icon_finalize(GObject *obj)
 {
     XfdesktopWindowIcon *icon = XFDESKTOP_WINDOW_ICON(obj);
     gchar data_name[256];
-    gint16 row, col;
-
+    guint16 row, col;
+    
+    if(icon->priv->pix)
+        g_object_unref(G_OBJECT(icon->priv->pix));
+    
     g_free(icon->priv->label);
-
+    
     /* save previous position */
     if(xfdesktop_icon_get_position(XFDESKTOP_ICON(icon), &row, &col)) {
         g_snprintf(data_name, 256, "--xfdesktop-last-row-%d",
@@ -100,14 +108,14 @@ xfdesktop_window_icon_finalize(GObject *obj)
         g_object_set_data(G_OBJECT(icon->priv->window),
                           data_name, GUINT_TO_POINTER(col + 1));
     }
-
+    
     g_signal_handlers_disconnect_by_func(G_OBJECT(icon->priv->window),
                                          G_CALLBACK(xfdesktop_window_name_changed_cb),
                                          icon);
     g_signal_handlers_disconnect_by_func(G_OBJECT(icon->priv->window),
                                          G_CALLBACK(xfdesktop_window_icon_changed_cb),
                                          icon);
-
+    
     G_OBJECT_CLASS(xfdesktop_window_icon_parent_class)->finalize(obj);
 }
 
@@ -118,10 +126,10 @@ xfdesktop_window_name_changed_cb(WnckWindow *window,
                                  gpointer user_data)
 {
     XfdesktopWindowIcon *icon = user_data;
-
+    
     g_free(icon->priv->label);
     icon->priv->label = NULL;
-
+    
     xfdesktop_icon_label_changed(XFDESKTOP_ICON(icon));
 }
 
@@ -130,54 +138,64 @@ xfdesktop_window_icon_changed_cb(WnckWindow *window,
                                  gpointer user_data)
 {
     XfdesktopWindowIcon *icon = user_data;
-
-    xfdesktop_icon_invalidate_pixbuf(XFDESKTOP_ICON(icon));
+    
+    if(icon->priv->pix) {
+        g_object_unref(G_OBJECT(icon->priv->pix));
+        icon->priv->pix = NULL;
+    }
+    
     xfdesktop_icon_label_changed(XFDESKTOP_ICON(icon));
 }
 
+
+
+
+
 static GdkPixbuf *
 xfdesktop_window_icon_peek_pixbuf(XfdesktopIcon *icon,
-                                 gint width, gint height)
+                                 gint size)
 {
     XfdesktopWindowIcon *window_icon = XFDESKTOP_WINDOW_ICON(icon);
-    GdkPixbuf *pix = NULL;
-
-    pix = wnck_window_get_icon(window_icon->priv->window);
-    if(pix) {
-        if(gdk_pixbuf_get_height(pix) != height) {
-            pix = gdk_pixbuf_scale_simple(pix, height, height, GDK_INTERP_BILINEAR);
-        } else
-            g_object_ref(G_OBJECT(pix));
+    
+    if(!window_icon->priv->pix || window_icon->priv->cur_pix_size != size) {
+        if(window_icon->priv->pix)
+            g_object_unref(G_OBJECT(window_icon->priv->pix));
+        
+        window_icon->priv->pix = wnck_window_get_icon(window_icon->priv->window);
+        if(window_icon->priv->pix) {
+            if(gdk_pixbuf_get_width(window_icon->priv->pix) != size) {
+                window_icon->priv->pix = gdk_pixbuf_scale_simple(window_icon->priv->pix,
+                                                                 size,
+                                                                 size,
+                                                                 GDK_INTERP_BILINEAR);
+            } else
+                g_object_ref(G_OBJECT(window_icon->priv->pix));
+            window_icon->priv->cur_pix_size = size;
+        }
     }
-
-    return pix;
+    
+    return window_icon->priv->pix;
 }
 
-static const gchar *
+static G_CONST_RETURN gchar *
 xfdesktop_window_icon_peek_label(XfdesktopIcon *icon)
 {
     XfdesktopWindowIcon *window_icon = XFDESKTOP_WINDOW_ICON(icon);
-
+    
     if(!window_icon->priv->label)
         window_icon->priv->label = g_strdup(wnck_window_get_name(window_icon->priv->window));
-
+    
     return window_icon->priv->label;
 }
 
-static gchar *
-xfdesktop_window_icon_get_identifier(XfdesktopIcon *icon)
-{
-    return g_strdup(xfdesktop_window_icon_peek_label(icon));
-}
-
 static gboolean
 xfdesktop_window_icon_activated(XfdesktopIcon *icon)
 {
     XfdesktopWindowIcon *window_icon = XFDESKTOP_WINDOW_ICON(icon);
-
+    
     wnck_window_activate(window_icon->priv->window,
                          gtk_get_current_event_time());
-
+    
     return TRUE;
 }
 
@@ -186,15 +204,24 @@ xfdesktop_window_icon_populate_context_menu(XfdesktopIcon *icon,
                                             GtkWidget *menu)
 {
     XfdesktopWindowIcon *window_icon = XFDESKTOP_WINDOW_ICON(icon);
-    GtkWidget *amenu = wnck_action_menu_new(window_icon->priv->window);
-    GtkWidget *mi, *img;
-
-    img = gtk_image_new_from_icon_name("", GTK_ICON_SIZE_MENU);
-    mi = xfdesktop_menu_create_menu_item_with_mnemonic(_("_Window Actions"), img);
-    gtk_menu_item_set_submenu (GTK_MENU_ITEM(mi), amenu);
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
-    gtk_widget_show(mi);
-
+    GtkWidget *amenu = wnck_create_window_action_menu(window_icon->priv->window);
+    GList *items, *l;
+    
+    /* this is unfortunately slightly retarded */
+    items = gtk_container_get_children(GTK_CONTAINER(amenu));
+    for(l = items; l; l = l->next) {
+        GtkWidget *mi = l->data;
+        g_object_ref(G_OBJECT(mi));
+        gtk_container_remove(GTK_CONTAINER(amenu), mi);
+        gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
+        g_object_unref(G_OBJECT(mi));
+    }
+    g_list_free(items);
+    gtk_widget_destroy(amenu);
+    
+    if(!items)
+        return FALSE;
+    
     return TRUE;
 }
 
@@ -207,10 +234,10 @@ xfdesktop_window_icon_new(WnckWindow *window,
     XfdesktopWindowIcon *icon = g_object_new(XFDESKTOP_TYPE_WINDOW_ICON, NULL);
     gchar data_name[256];
     gint row, col;
-
+    
     icon->priv->window = window;
     icon->priv->workspace = workspace;
-
+    
     /* check for availability of old position (if any) */
     g_snprintf(data_name, 256, "--xfdesktop-last-row-%d", workspace);
     row = GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(window),
@@ -220,14 +247,14 @@ xfdesktop_window_icon_new(WnckWindow *window,
                                              data_name));
     if(row > 0 && col > 0)
         xfdesktop_icon_set_position(XFDESKTOP_ICON(icon), row - 1, col - 1);
-
+    
     g_signal_connect(G_OBJECT(window), "name-changed",
                      G_CALLBACK(xfdesktop_window_name_changed_cb),
                      icon);
     g_signal_connect(G_OBJECT(window), "icon-changed",
                      G_CALLBACK(xfdesktop_window_icon_changed_cb),
                      icon);
-
+    
     return icon;
 }
 
